/*
**                    ---  term.c ---
**
**  EXAMPLE CODE: Terminal emulator. Can transfer files using
**  XMODEM and YMODEM protocols.
**
**  Link with TERM_IO, MODEM_IO, CRC, DOS, XYMODEM, and XYPACKET.
**  See TERM makefiles.
**
**  This example program (not the PCL4C library) is donated to
**  the Public Domain by MarshallSoft Computing, Inc. It is
**  provided as an example of the use of the PCL4C.
**
*/

#include <stdio.h>
#include <fcntl.h>
#include <sys\types.h>
#include <sys\stat.h>

#define FALSE 0
#define TRUE !FALSE
#define ONESECOND 18
#define NORMAL 0x07
#define INVERSE 0x70
#define MESSAGE_POS 48

/* Set HAYES to 1 if using a HAYES modem, else set HAYES to 0 */
#define HAYES 0

#include "pcl4c.h"
#include "ascii.h"

/*** Global variables ***/

int Port;               /* current COM port [0..3] */
char Filename[15];      /* file name buffer */
char Buffer[1024];      /* block buffer */
char  RxBuf[2048];
char *BaudRate[10] =  {"300","600","1200","2400","4800","9600",
                       "19200","38400","57600","115200"};
int BaudCode;         /* baud rate code ( index into BaudRate[] ) */
int CRCflag   = TRUE;
int OneKflag  = FALSE;
int BatchFlag = FALSE;
char Protocol = 'X';

/*** main program ***/

main(argc,argv)
int argc;
char *argv[];
{int DataFlag = FALSE;
 char c;
 int i, k;
 int n, rc;
 int Delta;         /* delta port status */
 int Status;        /* port status */
 char temp[81];
 /* right number of parameters ? */
 if(argc!=3)
     {printf("Usage: 'TERM port baud' -- example 'TERM 1 9600'\n");
      exit(1);
     }
 /* get port number from command line */
 Port = atoi(argv[1]) - 1;
 if((Port<0) || (Port>3))
     {printf("Port must be 1 to 4\n");
      exit(1);
     }
 /* get baud rate from command line */
 BaudCode = BaudMatch(argv[2]);
 if(BaudCode<0)
     {printf("Cannot recognize baud rate = %s\n",argv[2]);
      exit(1);
     }
 /* setup receive buffer */
 rc = SioRxBuf(Port,RxBuf,Size2048);
 if(rc<0)
     {SioError(rc);
      exit(1);
     }
 /* set parms & reset (initialize) COM port */
 SioParms(Port,NoParity,OneStopBit,WordLength8);
 MyStart(Port,BaudCode);
 /* init CRCR table */
 InitCRC();
 /* set DTR and RTS */
 SioDTR(Port,'S');
 SioRTS(Port,'S');
 /* initialize screen */
 Scroll(0,0,24,79,0,NORMAL);
 /* display status message */
 sprintf(temp," COM%d %s %c 'ESC for menu' ",1+Port,BaudRate[BaudCode],Protocol);
 for(i=0;i<strlen(temp);i++)
     {/* write character */
      Position(24,MESSAGE_POS+i);
      AttrWrite(temp[i],INVERSE);
     }
 /* home cursor & clear CCL receive buffer */
 Position(0,0);
 SioRxFlush(Port);

#if HAYES
 /* wait for Modem to say its ready */
 printf("Waiting for Modem DSR.");
 while( !SioDSR(Port) )
     {
      if(SioKeyPress()||SioBrkKey()) MyExit(0,"Aborted by user");
      putchar('.');
      SioDelay(18);
     }
 putchar('\n');
 /* initialize (Hayes compatible) modem */
 SendTo(Port,"!AT E1 S7=60 S11=60 V1 X1 Q0 S0=1!");
 if(!WaitFor(Port,"OK")) MyExit(0,"Expecting OK");
 printf("\nMODEM READY\n");
#endif

 /* enter terminal loop */
 SioRxFlush(Port);
 while(1)
     {/* Control-BREAK ? */
      if(SioBrkKey()) MyExit(0,"User pressed Ctrl-BREAK");
      /* was key pressed ? */
      if(SioKeyPress())
          {/* read key press */
           i = SioKeyRead();
           if((char)i==ESC)
               {/* process user's request */
                ProcessESC();
                continue;
               }
           else PutChar(Port,i);
          }
      /* was break detected ? */
      if( SioBrkSig(Port,'D') ) DisplayLine("BREAK detected ",NULL,0);
      /* any incoming over serial port ? */
      i = GetChar(Port,0);
      if(i>-1)
          {/* good character */
           if(DataFlag&(i<0x20))
               {MyCrtWrite('^');
                MyCrtWrite('@'+i);
               }
           else MyCrtWrite(i&0x007f);
          }
      /* any change in DCD or DSR ? */
      Delta = SioModem(Port,DeltaDCD|DeltaDSR);
      if(Delta)
          {/* display new status */
           Status = SioModem(Port,DCD|DSR);
           if(!Delta&DeltaDCD) MyExit(0,"Dropped DSD");
           if(!Delta&DeltaDSR) MyExit(0,"Dropped DSR");
          }
     } /* end -- key pressed */
}

/*** write to screen except for bottom line ***/

int MyCrtWrite(ch)
char ch;
{/* write character */
 SioCrtWrite(ch);
 /* scroll all but bottom line */
 if(GetRow()==24)
    {Scroll(0,0,23,79,1,NORMAL);
     Position(23,0);
    }
}

/*** make multiple attempts to reset port ***/

int MyStart(Port,BaudCode)
int Port;
int BaudCode;
{int i, rc;
 /* try up to 3 times to reset COM port */
 for(i=0;i<3;i++)
     {printf("Resetting COM%d at %s baud\n",Port+1,BaudRate[BaudCode]);
      if( (rc = SioReset(Port,BaudCode))==0) return;
      if(rc<0) MyExit(rc,"Error resetting port");
      SioDone(Port);
      /* display errors */
      if(rc&OverrunError) puts("Overrun Error");
      if(rc&ParityError)  puts("Parity Error");
      if(rc&FramingError) puts("Framing Error");
      if(rc&BreakDetect)  puts("Break Detect");
     }
 exit(1);
}

/*** find baud rate string in table ***/

int BaudMatch(ptr)
char *ptr;
{int i;
 /* find baud rate in table */
 for(i=0;i<10;i++) if(strcmp(BaudRate[i],ptr)==0) return(i);
 return(-1);
}

/*** user pressed Escape */

int ProcessESC()
{int i;
 int rc;
 int c1, c2;
 char Answer[2]; /* array for 1 char answer */
 int row, col;
 /* user pressed <ESC> */
 Answer[0] = '?';
 Answer[1] = '\0';
 DisplayLine("Q)uit P)rotocol S)end R)eceive: ",Answer,1);
 if(strlen(Answer)) switch(toupper(Answer[0]))
     {
      case 'P':
          DisplayLine("X)MODEM Y)MODEM: ",Answer,1);
          if(strlen(Answer)) switch( toupper(Answer[0]) )
               {
                case 'X':
                    Protocol = 'X';
                    ShowProtocol();
                    OneKflag = FALSE;
                    BatchFlag = FALSE;
                    CRCflag = TRUE;
                    DisplayLine("Protocol = XMODEM",NULL,1);
                    break;
                case 'Y':
                    Protocol = 'Y';
                    ShowProtocol();
                    OneKflag = TRUE;
                    BatchFlag = TRUE;
                    CRCflag = TRUE;
                    DisplayLine("Protocol = YMODEM",NULL,1);
                    break;
                default:
                    DisplayLine("Must answer X or Y",NULL,1);
                    break;
               }
          break;
      case 'Q':
          MyExit(0,"User pressed ESC");
          break;
      case 'R':
          /* XMODEM / YMODEM receive */
          if(BatchFlag)
               {do
                    {/* receive files till get empty filename */
                     RxyModem(Port,Filename,Buffer,CRCflag,BatchFlag);
                     if(SioKeyPress()) break;
                    } while(Filename[0]!='\0');
               }
          else /* not Batch */
               {DisplayLine("Enter filename:",Filename,15);
                if(strlen(Filename)==0) break;
                RxyModem(Port,Filename,Buffer,CRCflag,BatchFlag);
               }
          break;
      case 'S':
          DisplayLine("Enter filename:",Filename,15);
          if(strlen(Filename)==0) break;
          /* XMODEM / YMODEM send */
          TxyModem(Port,Filename,Buffer,OneKflag,BatchFlag);
          if(BatchFlag)
               {/* send empty filename */
                Filename[0] = '\0';
                SioDelay(5);
                TxyModem(Port,Filename,Buffer,OneKflag,BatchFlag);
               }
          break;
     default:
          DisplayLine("Must answer Q, P, S, or R",NULL,0);
          break;
    } /* end switch */
}

/*** show protocol choosen ***/
int ShowProtocol()
{int Row;
 int Col;
 Row = GetRow();
 Col = GetCol();
 Position(24,MESSAGE_POS+strlen(BaudRate[BaudCode])+7);
 AttrWrite(Protocol,INVERSE);
 Position(Row,Col);
}

/*** exit program ***/

int MyExit(code,msg)
int code;
char *msg;
{int rc;
 if(code<0) SioError(code);
 /* Assert UART break & sign off */
 SioBrkSig(Port,'A');
 printf("\nTERMINATING: %s\n",msg);
 SioDelay(ONESECOND);
 SioDone(Port);
 exit(0);
}